/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import java.io.File;


/**
 * This class is derived from Server class and is the base class for all classes representing data server to be used
 * while running tests. Those classes will have corresponding elements in the Ant build file. This class is a base for
 * another component which will provide testing ability for database servers. TestCase classes should also be provided
 * by that component.
 * <p/>
 * <b>Supported attributes:</b> <ul><li><tt>jarfile</tt> - the implementation defines if is required</li></ul>
 * <p/>
 * This class is mutable, i.e. not thread-safe.
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public abstract class DataServer extends Server {

    /**
     * This field represents the <tt>jarFile</tt> attribute, which specifies the JAR archive which contains data needed
     * for data server initialization. Derived classes should determine if this attribute is required. The field is
     * initialized to <tt>null</tt> and set via the {@link #setJarFile(java.io.File)} method.
     */
    private File jarFile = null;

    /**
     * Creates a DataServer instance. The type of server represented by the class is specified. Type is a string like
     * "mysql4x", which actually specifies the name and version of the server.
     *
     * @param type the type of the server
     *
     * @throws org.apache.tools.ant.BuildException
     *                                  if the type of the server is unrecognized, or any other error happens
     * @throws IllegalArgumentException if type is <tt>null</tt>
     */
    protected DataServer(final String type) {
        //arg checking done in super constructor
        super(type);
    }

    /**
     * This method sets the value of the <tt>jarfile</tt> attribute, which specifies the JAR archive which contains data
     * needed for data server initialization.
     *
     * @param jarFile the value of the <tt>jarfile</tt> attribute to be set
     */
    public void setJarFile(final File jarFile) {
        this.jarFile = jarFile;
    }

    /**
     * This method returns the value of the <tt>jarfile</tt> attribute, which specifies the JAR archive which contains
     * data needed for data server initialization.
     *
     * @return the value of the <tt>jarfile</tt> attribute
     */
    public File getJarFile() {
        return jarFile;
    }
}
