/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import org.apache.cactus.integration.ant.CactifyEarTask;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;

import java.io.File;


/**
 * This class represents an Ant task that injects elements necessary to run the tests into an existing EAR file. It can
 * be used to prepare EARs for {@link DefaultWebApplicationServer} class.
 * <p/>
 * This class is mutable, i.e. not thread-safe.
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class PrepareTestEarTask extends Task {

    /**
     * This field stores a wrapped {@link org.apache.cactus.integration.ant.CactifyEarTask} instance. It is initialized
     * in field initializer. The instance in this field is the delegate to which all implementation and functionality is
     * delegated.
     */
    private final CactifyEarTask cactifyEarTask = new CactifyEarTask();

    /**
     * Creates a new PrepareTestEarTask.
     */
    public PrepareTestEarTask() {
        // This constructor is intentionally empty
    }

    /**
     * This method sets the value of <tt>srcfile</tt> attribute, which specifies the source archive to inject the
     * elements required to run tests into.
     *
     * @param srcFile the source archive to inject the elements required to run tests into
     */
    public void setSrcFile(final File srcFile) {
        cactifyEarTask.setSrcFile(srcFile);
    }

    /**
     * This method sets the value of <tt>destfile</tt> attribute, which specifies the destination file, archive with the
     * injected elements required to run tests will be written to.
     *
     * @param destFile the destination file archive with the injected elements required to run tests will be written to
     */
    public void setDestFile(final File destFile) {
        cactifyEarTask.setDestFile(destFile);
    }

    /**
     * This method adds a fileset to the preparation task. The fileset is used to specify additional files that are to
     * be included in the prepared ear file.
     *
     * @param set the fileset specifying the files to be added
     */
    public void addFileset(final FileSet set) {
        cactifyEarTask.addFileset(set);
    }

    /**
     * This method runs the preparation task.
     *
     * @throws org.apache.tools.ant.BuildException
     *          if any error happens
     */
    public void execute() {
        cactifyEarTask.setProject(getProject());
        try {
            cactifyEarTask.execute();
        } catch (Exception e) {
            // Catch of Exception is normally discouraged by TC style, but as we want
            // to 'shield' the caller from the internals of the wrapped implementation
            // of cactus (which sometimes can throw NPEs) we simply wrap everything
            // into BuildException to assure normal ant behavior
            throw new BuildException("Error while preparing the ear file.", e);
        }
    }
}
