/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import junit.framework.TestCase;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Target;

import java.io.File;


/**
 * This is an abstract test class that contains the test cases for the functionality declare din {@link Server}.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public abstract class AbstractServerTest extends TestCase {
    /**
     * This is a valid type name used in unit tests.
     */
    protected static final String TYPE = "tomcat5x";
    /**
     * This is an AntTaskInfo instance used in tests, created in {@link #setUp()}.
     */
    private AntTaskInfo info;
    /**
     * This is the tested instance, created in {@link #setUp()}.
     */
    private Server server;

    /**
     * This method tests {@link Server#Server(String)} for correctness.
     */
    public void testCtor() {
        final Server instance = createInstance(TYPE);
        assertEquals("constant field value changed", "user_name", Server.USER_NAME);
        assertEquals("constant field value changed", "password", Server.PASSWORD);
    }

    /**
     * This method tests the correct failure behavior of {@link Server#Server(String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> type given
     */
    public void testCtorFailureNullType() {
        try {
            createInstance(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link Server#setDir(java.io.File)}  and {@link Server#getDir()}for correctness.
     */
    public void testGetSetDir() {
        assertEquals("unexpected initial value", null, server.getDir());
        final File dir = new File(".");
        server.setDir(dir);
        assertEquals("setter does not work", dir, server.getDir());
    }

    /**
     * This method tests {@link Server#setPort(int)}  and {@link Server#getPort()}for correctness.
     */
    public void testGetSetPort() {
        assertEquals("unexpected initial value", -1, server.getPort());
        final int port = 1234;
        server.setPort(port);
        assertEquals("setter does not work", port, server.getPort());
    }

    /**
     * This method tests the correct failure behavior of {@link Server#setPort(int)}.
     * <p/>
     * <b>Failure reason</b>: non-positive port given
     */
    public void testSetPortFailNonPositivePort() {
        try {
            server.setPort(-1);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link Server#setServer(String)}  and {@link Server#getServer()}for correctness.
     */
    public void testGetSetServer() {
        assertEquals("unexpected initial value", "localhost", server.getServer());
        final String val = "server";
        server.setServer(val);
        assertEquals("setter does not work", val, server.getServer());
    }

    /**
     * This method tests the correct failure behavior of {@link Server#setServer(String)}.
     * <p/>
     * <b>Failure reason</b>: empty String server given
     */
    public void testSetServerFailEmptyServer() {
        try {
            server.setServer("   ");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link Server#setStartServer(boolean)}  and {@link Server#getStartServer()}for correctness.
     */
    public void testGetSetStartServer() {
        assertEquals("unexpected initial value", true, server.getStartServer());
        server.setStartServer(false);
        assertEquals("setter does not work", false, server.getStartServer());
    }

    /**
     * This method tests {@link Server#setInitServer(boolean)}  and {@link Server#getInitServer()}for correctness.
     */
    public void testGetSetInitServer() {
        assertEquals("unexpected initial value", true, server.getInitServer());
        server.setInitServer(false);
        assertEquals("setter does not work", false, server.getInitServer());
    }

    /**
     * This method tests {@link Server#setAntTaskInfo(AntTaskInfo)} and {@link Server#getAntTaskInfo()}for correctness.
     */
    public void testGetSetAntTaskInfo() {
        assertEquals("unexpected initial value", null, server.getAntTaskInfo());
        server.setAntTaskInfo(info);
        assertEquals("setter does not work", info, server.getAntTaskInfo());
    }

    /**
     * This method tests the correct failure behavior of {@link Server#setAntTaskInfo(AntTaskInfo)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> AntTaskInfo given
     */
    public void testSetAntTaskInfoFailNullAntTaskInfo() {
        try {
            server.setAntTaskInfo(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link Server#createCredentials()}  and {@link Server#getCredentials()}for correctness.
     */
    public void testCreateGetCredentials() {
        assertEquals("unexpected initial value", null, server.getCredentials());
        final CredentialsElement element = server.createCredentials();
        assertEquals("setter does not work", element, server.getCredentials());
    }

    /**
     * This method tests the correct failure behavior of {@link Server#createCredentials()}.
     * <p/>
     * <b>Failure reason</b>: duplicate call to the method
     */
    public void testCreateCredentialdsFailDuplicate() {
        server.createCredentials();
        try {
            server.createCredentials();
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        info = new AntTaskInfo(new Project(), "taskName", Location.UNKNOWN_LOCATION, new Target());
        final String type = TYPE;
        server = createInstance(type);
    }

    /**
     * This method creates the instance to be tested by calling the constructor using the given type value as argument.
     *
     * @param type the type value to be used as constructor argument
     *
     * @return the instance created
     */
    protected abstract Server createInstance(String type);
}
