/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import junit.framework.TestCase;

import java.util.Map;


/**
 * This class contains the unit tests for {@link com.topcoder.testframework.CredentialsElement}.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class CredentialsElementTest extends TestCase {
    /**
     * This is a name used in test cases.
     */
    private static final String NAME = "XXName";
    /**
     * This is a value used in test cases.
     */
    private static final String VALUE = "value";
    /**
     * This is the instance to be tested, initialized in {@link #setUp()}.
     */
    private CredentialsElement element;

    /**
     * This method tests {@link CredentialsElement#CredentialsElement()} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCtor() throws Exception {
        final CredentialsElement instance = new CredentialsElement();
        assertNotNull(instance.getPropertyMap());
        assertEquals(0, instance.getPropertyMap().size());
    }

    /**
     * This method tests {@link CredentialsElement#createDynamicElement(String)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElement() throws Exception {
        final Object o = element.createDynamicElement(NAME);

        assertEquals(CredentialProperty.class, o.getClass());
        final CredentialProperty credentialProperty = (CredentialProperty) o;
        assertEquals(NAME, TestingUtils.getFieldValue(credentialProperty, "name"));
        final Map map = element.getPropertyMap();
        assertEquals(0, map.size());
        credentialProperty.setValue(VALUE);
        assertEquals(VALUE, map.get(NAME));
    }

    /**
     * This method tests the correct failure behavior of {@link CredentialsElement#createDynamicElement(String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> name given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElementNullName() throws Exception {
        try {
            element.createDynamicElement(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link CredentialsElement#createDynamicElement(String)}.
     * <p/>
     * <b>Failure reason</b>: empty name given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElementEmptyName() throws Exception {
        try {
            element.createDynamicElement("   ");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link com.topcoder.testframework.CredentialsElement#getPropertyMap()} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetPropertyMap() throws Exception {
        final Map map = element.getPropertyMap();
        assertEquals(0, map.size());

        ((CredentialProperty) element.createDynamicElement(NAME)).setValue(VALUE);

        assertEquals(VALUE, map.get(NAME));
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        element = new CredentialsElement();
    }
}
