/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import com.topcoder.testframework.TestingUtils;
import junit.framework.TestCase;
import org.apache.cactus.integration.ant.CactifyEarTask;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.types.FileSet;

import java.io.File;
import java.util.Vector;


/**
 * This class contains the unit tests for {@link com.topcoder.testframework.web.PrepareTestEarTask}.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class PrepareTestEarTaskTest extends TestCase {
    /**
     * This is the instance to be tested, initialized in {@link #setUp()}.
     */
    private PrepareTestEarTask task;
    /**
     * This is the private cactifyEarTask field of the {@link #task}.
     */
    private CactifyEarTask cactifyEarTask;

    /**
     * This method tests {@link PrepareTestEarTask#PrepareTestEarTask()} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCtor() throws Exception {
        final PrepareTestEarTask instance = new PrepareTestEarTask();
    }

    /**
     * This method tests {@link PrepareTestEarTask#setSrcFile(java.io.File)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testSetSrcFile() throws Exception {

        assertNull(TestingUtils.getFieldValue(cactifyEarTask, "srcFile"));
        final File file = new File(".");
        task.setSrcFile(file);
        assertEquals(file, TestingUtils.getFieldValue(cactifyEarTask, "srcFile"));
        task.setSrcFile(null);
        assertNull(TestingUtils.getFieldValue(cactifyEarTask, "srcFile"));
    }

    /**
     * This method tests {@link PrepareTestEarTask#setDestFile(java.io.File)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testSetDestFile() throws Exception {

        assertNull(cactifyEarTask.getDestFile());
        final File file = new File(".");
        task.setDestFile(file);
        assertEquals(file, cactifyEarTask.getDestFile());
        task.setDestFile(null);
        assertNull(cactifyEarTask.getDestFile());
    }

    /**
     * This method tests {@link PrepareTestEarTask#addFileset(org.apache.tools.ant.types.FileSet)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddFileset() throws Exception {

        final Vector tasksFileSets = (Vector) TestingUtils.getFieldValue(cactifyEarTask, "filesets");

        assertEquals(0, tasksFileSets.size());
        final FileSet fileset = new FileSet();
        task.addFileset(fileset);
        assertEquals(1, tasksFileSets.size());
        assertTrue(tasksFileSets.contains(fileset));
    }

    /**
     * This method tests {@link PrepareTestEarTask#execute()} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testExecute() throws Exception {

        task.setSrcFile(new File(TestingUtils.VALID_UNPREPARED_EAR_FILE));
        final File destFile = new File("test_files/dummyFiles/built.ear");
        if (destFile.exists()) {
            destFile.delete();
        }
        task.setDestFile(destFile);
        task.execute();

        assertTrue("destination file was not built", destFile.exists() && destFile.length() > 0);
        destFile.delete();

    }

    /**
     * This method tests the correct failure behavior of {@link PrepareTestEarTask#execute()}.
     * <p/>
     * <b>Failure reason</b>: srcFile has not been specified
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testExecuteFailNoSrcFileSet() throws Exception {
        task.setDestFile(new File("test_files/dummyFiles/built.ear"));
        try {
            task.execute();
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link PrepareTestEarTask#execute()}.
     * <p/>
     * <b>Failure reason</b>: srcFile specified is no valid file
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testExecuteFailInvalidSrcFileSet() throws Exception {
        task.setSrcFile(new File("."));
        task.setDestFile(new File("test_files/dummyFiles/built.ear"));
        try {
            task.execute();
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link PrepareTestEarTask#execute()}.
     * <p/>
     * <b>Failure reason</b>: destFile has not been specified
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testExecuteFailNoDestFileSet() throws Exception {
        task.setSrcFile(new File(TestingUtils.VALID_UNPREPARED_EAR_FILE));
        try {
            task.execute();
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        task = new PrepareTestEarTask();
        final Project project = new Project();
        project.init();
        task.setProject(project);
        cactifyEarTask = (CactifyEarTask) TestingUtils.getFieldValue(task, "cactifyEarTask");

    }
}
