/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import com.topcoder.testframework.TestingUtils;
import junit.framework.TestCase;
import org.apache.cactus.integration.ant.CactifyWarTask;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.ZipFileSet;

import java.io.File;
import java.util.Vector;


/**
 * This class contains the unit tests for {@link PrepareTestWarTask}.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class PrepareTestWarTaskTest extends TestCase {
    /**
     * This is the instance to be tested, initialized in {@link #setUp()}.
     */
    private PrepareTestWarTask task;
    /**
     * This is the private cactifyWarTask field of the {@link #task}.
     */
    private CactifyWarTask cactifyWarTask;

    /**
     * This method tests {@link PrepareTestWarTask#PrepareTestWarTask()} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCtor() throws Exception {
        final PrepareTestWarTask instance = new PrepareTestWarTask();
    }

    /**
     * This method tests {@link PrepareTestWarTask#setSrcFile(java.io.File)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testSetSrcFile() throws Exception {

        assertNull(TestingUtils.getFieldValue(cactifyWarTask, "srcFile"));
        final File file = new File(".");
        task.setSrcFile(file);
        assertEquals(file, TestingUtils.getFieldValue(cactifyWarTask, "srcFile"));
        task.setSrcFile(null);
        assertNull(TestingUtils.getFieldValue(cactifyWarTask, "srcFile"));
    }

    /**
     * This method tests {@link PrepareTestWarTask#setDestFile(java.io.File)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testSetDestFile() throws Exception {

        assertNull(cactifyWarTask.getDestFile());
        final File file = new File(".");
        task.setDestFile(file);
        assertEquals(file, cactifyWarTask.getDestFile());
        task.setDestFile(null);
        assertNull(cactifyWarTask.getDestFile());
    }

    /**
     * This method tests {@link PrepareTestWarTask#addFileset(org.apache.tools.ant.types.FileSet)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddFileset() throws Exception {

        final Vector tasksFileSets = (Vector) TestingUtils.getFieldValue(cactifyWarTask, "filesets");

        assertEquals(0, tasksFileSets.size());
        final FileSet fileset = new FileSet();
        task.addFileset(fileset);
        assertEquals(1, tasksFileSets.size());
        assertTrue(tasksFileSets.contains(fileset));
    }

    /**
     * This method tests {@link PrepareTestWarTask#addLib(org.apache.tools.ant.types.ZipFileSet)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddLib() throws Exception {

        final Vector tasksFileSets = (Vector) TestingUtils.getFieldValue(cactifyWarTask, "filesets");

        assertEquals(0, tasksFileSets.size());
        final ZipFileSet fileset = new ZipFileSet();
        task.addLib(fileset);
        assertEquals(1, tasksFileSets.size());
        assertTrue(tasksFileSets.contains(fileset));
        assertEquals("WEB-INF/lib/", fileset.getPrefix(null));
    }

    /**
     * This method tests {@link PrepareTestWarTask#addClasses(org.apache.tools.ant.types.ZipFileSet)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddClasses() throws Exception {

        final Vector tasksFileSets = (Vector) TestingUtils.getFieldValue(cactifyWarTask, "filesets");

        assertEquals(0, tasksFileSets.size());
        final ZipFileSet fileset = new ZipFileSet();
        task.addClasses(fileset);
        assertEquals(1, tasksFileSets.size());
        assertTrue(tasksFileSets.contains(fileset));
        assertEquals("WEB-INF/classes/", fileset.getPrefix(null));
    }

    /**
     * This method tests {@link PrepareTestWarTask#execute()} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testExecute() throws Exception {

        task.setSrcFile(new File(TestingUtils.VALID_UNPREPARED_WAR_FILE));
        final File destFile = new File("test_files/dummyFiles/built.war");
        if (destFile.exists()) {
            destFile.delete();
        }
        task.setDestFile(destFile);
        task.execute();

        assertTrue("destination file was not built", destFile.exists() && destFile.length() > 0);
        destFile.delete();

    }

    /**
     * This method tests the correct failure behavior of {@link PrepareTestWarTask#execute()}.
     * <p/>
     * <b>Failure reason</b>: srcFile has not been specified
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testExecuteFailNoSrcFileSet() throws Exception {
        task.setDestFile(new File("test_files/dummyFiles/built.war"));
        try {
            task.execute();
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link PrepareTestWarTask#execute()}.
     * <p/>
     * <b>Failure reason</b>: srcFile specified is no valid file
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testExecuteFailInvalidSrcFileSet() throws Exception {
        task.setSrcFile(new File("."));
        task.setDestFile(new File("test_files/dummyFiles/built.war"));
        try {
            task.execute();
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link PrepareTestWarTask#execute()}.
     * <p/>
     * <b>Failure reason</b>: destFile has not been specified
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testExecuteFailNoDestFileSet() throws Exception {
        task.setSrcFile(new File(TestingUtils.VALID_UNPREPARED_WAR_FILE));
        try {
            task.execute();
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        task = new PrepareTestWarTask();
        final Project project = new Project();
        project.init();
        task.setProject(project);
        cactifyWarTask = (CactifyWarTask) TestingUtils.getFieldValue(task, "cactifyWarTask");

    }
}
